from typing import Callable, Tuple

import torch
from torch import nn


Hook = Callable[[nn.Module, Tuple[torch.Tensor, ...], torch.Tensor], None]
HookPair = Tuple[nn.Module, Hook]


class Hooker:

    def __init__(self, *layer_hook_pairs: HookPair) -> None:
        self._layer_hook_pairs = layer_hook_pairs
        self._hook_handles = None

    def __enter__(self):
        assert self._hook_handles is None, "Multiple enter detected!"
        self._hook_handles = [layer.register_forward_hook(hook) for layer, hook in self._layer_hook_pairs]

    def __exit__(self, exc_type, exc_value, traceback):
        assert self._hook_handles is not None, "Calling exit without entering!"
        [handle.remove() for handle in self._hook_handles]
        self._hook_handles = None
    